
#include "acl-admins.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct acl_admin_elem {
    char *acl_admin;
    struct acl_admin_elem *next;
};

static struct acl_admin_elem *first_elem;

int
is_acl_admin (char *username)
{
    struct acl_admin_elem *current = first_elem;

    while (current != NULL) {
	if (strcmp(current->acl_admin, username) == 0)
	    return 1;

	current = current->next;
    }

    return 0;
}

static int
internal_add_acl_admin (char *username)
{
    struct acl_admin_elem *new_elem;

    new_elem = (struct acl_admin_elem *) calloc(1, sizeof(struct acl_admin_elem));
    if (new_elem == NULL)
	return 0;

    new_elem->acl_admin = strdup(username);
    if (new_elem->acl_admin == NULL) {
	free(new_elem);
	return 0;
    }

    new_elem->next = first_elem;
    first_elem = new_elem;

    return 1;
}


#define MAX_LINE_SIZE 128

int
load_acl_admins_from_file (char *filename)
{
    FILE *file;
    char buf[MAX_LINE_SIZE];
    int lineno = 1;
    char *p;

    file = fopen(filename, "r");
    if (file == NULL)
	return -1;

    while ((p = fgets(buf, MAX_LINE_SIZE, file)) != NULL) {
	int len = strlen(buf);
	if (buf[len - 1] != '\n')
	    return lineno;

	buf[len - 1] = '\0';

	if (!internal_add_acl_admin(buf))
	    return lineno;

	lineno++;
    }

    if (fclose(file) == EOF)
	return -1;

    return 0;
}
