/* 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.  

   Copyright (C) Alexey Mahotkin <alexm@hsys.msk.ru> 2000

   Reads CVS user name, password and repository, using :pserver:
   protocol.

*/

#define _BSD_SOURCE 1

#include "config.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "cvs-server.h"
#include "login.h"
#include "fd.h"
#include "env.h"
#include "scramble.h"

char *usage = 
"Usage: %s /repos1 ... /reposN -- subprogram\n"
"This program is designed to be run from TCP-server like inetd\n\n";

#define BUFFER_SIZE 512

static void
i_hate_you (void)
{
    printf("I HATE YOU\n");
    fflush(stdout);
    exit(1);
}

int
main (int argc, char *argv[])
{
    char *buffer;
    int errorlevel;
    char **subprog;
    char *repository;
    char *username;
    char *password;
    int pi[2];
    pid_t pid;
    int wstat;
    int verify_only = 0;
    
    if (argc <= 3)
    {
	printf (usage, argv[0]);
	exit (EXIT_FAILURE);
    }

    subprog = argv + 1;
    while (subprog) {
	if (strcmp(*subprog, "--") == 0)
	    break;
	subprog++;
    }

    if (!subprog)
    {
	printf (usage, argv[0]);
	exit (EXIT_FAILURE);
    }
    
    subprog++;
    
    buffer = malloc(BUFFER_SIZE);
    if (!buffer)
	exit(1);

    get_buffer(buffer, BUFFER_SIZE);

    if (strcmp(buffer, "BEGIN AUTH REQUEST") != 0)
    {
	if (strcmp(buffer, "BEGIN VERIFICATION REQUEST") == 0)
	    verify_only = 1;
	else
	    exit(1);
    }

#define GET_NEXT(what)			\
					\
    get_buffer(buffer, BUFFER_SIZE);	\
					\
    what = strdup(buffer);		\
    if (!what)				\
	exit(1);			\

    GET_NEXT(repository);
    GET_NEXT(username);
    GET_NEXT(password);

    get_buffer(buffer, BUFFER_SIZE);

    if (verify_only)
    {
	if (strcmp(buffer, "END VERIFICATION REQUEST") != 0)
	    exit(1);
    } else
	if (strcmp(buffer, "END AUTH REQUEST") != 0)
	    exit(1);

    password = descramble (password);
    if (!password)
	i_hate_you ();

    strip_trailing_slash (repository);

    if (!repository_ok (repository, argc, argv))
	i_hate_you ();

    setenvvar("REPOSITORY", repository);
    setenvvar("CVSUSER", username);
    setenvvar("CVSSERVER", "pserver");
    if (strcmp(username, "anonymous") == 0)
	setenvvar("ANONYMOUS", password);

    if (!create_pipe (pi))
    {
	fprintf (stderr, "E %s: cannot create a pipe: %s\n", argv[0], strerror(errno));
	exit (1);
    }

    pid = fork ();

    if (pid == -1)
    {
	fprintf (stderr, "E %s: cannot fork(): %s\n", argv[0], strerror(errno));
	exit (1);
    }

    if (pid == 0)
    {
	close(pi[1]);
	execvp(*subprog, subprog);
	fprintf (stderr, "E %s: cannot exec(%s): %s\n", 
		 argv[0], *subprog, strerror(errno));
	_exit(1);
    }

    close(pi[0]);
    write(pi[1], username, strlen(username) + 1);
    write(pi[1], password, strlen(password) + 1);
    write(pi[1], repository, strlen(repository) + 1);
    close(pi[1]);
    memset(password, 0, strlen(password));

    if (waitpid(pid, &wstat, 0) == -1)
    {
	fprintf (stderr, "E %s: waitpid() returned -1: %s\n", argv[0], strerror(errno));
	exit (1);
    }

    if (WIFSIGNALED(wstat)) {
	fprintf (stderr, "E %s: %s crashed\n", argv[0], *subprog);
	exit (1);
    }

    errorlevel = WEXITSTATUS(wstat);

    if (errorlevel == 2)
	printf("E cvs-pserver: Internal error: %s misuse\n", *subprog);

    if ((errorlevel >= 111) && (errorlevel <= 120))
	printf("E cvs-pserver: Temporary %s error %d\n", *subprog, errorlevel);

    if (errorlevel != 0)
	i_hate_you ();

    exit (0);
}
