/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2000

*/

#include <assert.h>
#include <string.h>

#include "getline.h"

#include "cvspasswd-file.h"

/*

  Creates /repos/CVSROOT/cvspasswd file with proper permissions.
  Assumes to be protected by atexit(unlink_filename).

*/

FILE *
create_cvspasswd (filename)
    char *filename;
{
    int fd = open(filename, O_RDWR | O_CREAT|O_EXCL, 0640);
    if (fd == -1)
	return NULL;

    return fdopen(fd, "w");
}

/* 

   Scans 'cvspasswd' file and copies its lines to 'tmpcvspasswd'.  If
   'username' line is found, do something according to 'action':

   - find_action_error: print "already exists" and exit;

   - find_action_remove: skip this line and do not write it to
   'tmpcvspasswd', return (char *) 1;

   - find_action_return: return a pointer to line found;

   - find_action_none: just copy the line to 'tmpcvspasswd' as if
   nothing happened and continue;

*/

char*
find_user (cvspasswd, tmpcvspasswd, username, action)
    FILE *cvspasswd;
    FILE *tmpcvspasswd;
    char *username;
    enum find_user_action action;
{
    char *passwd_line = NULL;
    size_t passwd_line_allocated = 0;

    while (cvs_getline(&passwd_line, &passwd_line_allocated, cvspasswd) != -1)
    {
	char *field_user, *field_password;

	assert(strlen(passwd_line) > 0);
	passwd_line[strlen(passwd_line) - 1] = '\0'; /* strip trailing newline */
	field_user = strdup(passwd_line);
	if (!field_user) {
/*	    printf("450 %s: out of memory\n", argv[0]); */
	    return NULL;
	}

	field_password = strchr(field_user, ':');
	if (!field_password) {
/*	    printf ("450 %s: broken cvspasswd line for user %s\n", argv[0],username); */
	    return NULL;
	}

	*field_password = '\0';

	if (strcmp(field_user, username) == 0) {
	    switch (action)
	    {
	    case find_action_error:
/*		printf("450 %s: user %s already exists\n", argv[0], username); */
		exit(EXIT_FAILURE);
		break;
		
	    case find_action_remove:
		break;

	    case find_action_return:
		return passwd_line;
		break;

	    case find_action_do_none:
		fputs(passwd_line, tmpcvspasswd);
		putc('\n', tmpcvspasswd);
		break;
	    }
	}
	else
	{
	    fputs(passwd_line, tmpcvspasswd);
	    putc('\n', tmpcvspasswd);
	}
    }
    if (passwd_line_allocated)
      free(passwd_line);
    return NULL;
}
