/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Morozov <morozov@novosoft.ru>, Novosoft, Inc. 2001

   Network client using file descriptor

*/

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "network-client.h"
#include "fd-client.h"

struct fd_client_data {
    int fdin;
    int fdout;
};

typedef struct fd_client_data *FCD;

static NCLI_STATUS fd_connect(NCLI ncli);
static NCLI_STATUS fd_disconnect(NCLI ncli);
static int fd_read(NCLI ncli, char *buf, unsigned int len);
static NCLI_STATUS fd_write(NCLI ncli, char *buf, unsigned int len);
static NCLI_STATUS fd_flush(NCLI ncli);

NCLI
create_fd_client(int fdin, int fdout, cvsroot_t *CVSROOT, ncli_error_cb error_cb)
{
    FCD fcd;
    struct network_client *nc;
    (void) CVSROOT;

    nc = malloc(sizeof(struct network_client));
    if (nc == NULL)
	return NULL;

    fcd = malloc(sizeof(struct fd_client_data));
    if (fcd == NULL) {
	free(nc);
	return NULL;
    }

    fcd->fdin = fdin;
    fcd->fdout = fdout;

    nc->connect = fd_connect;
    nc->disconnect = fd_disconnect;
    nc->read = fd_read;
    nc->write = fd_write;
    nc->flush = fd_flush;
    nc->error = error_cb;
    nc->client_data = fcd;

    return nc;
}


static NCLI_STATUS
fd_connect(NCLI ncli)
{
    (void) ncli;
    /* do nothing */
    return NCLI_SUCCESS;
}

static NCLI_STATUS
fd_disconnect (NCLI ncli)
{
    FCD fcd = (FCD) ncli->client_data;
    if (close(fcd->fdin) == -1)
	return NCLI_FAILURE;

    if (close(fcd->fdout) == -1)
	return NCLI_FAILURE;
    return NCLI_SUCCESS;
}

static int
fd_read (NCLI ncli, char *buf, unsigned int len)
{
    FCD fcd = (FCD) ncli->client_data;
    return basic_fd_read(ncli, fcd->fdin, buf, len);
}

ssize_t
basic_fd_read(NCLI ncli, int fd, char* buf, size_t len)
{
    int n = read(fd, buf, len);
    if (n == -1) {
	if (errno == EAGAIN)
	    return -1;
	return ncli->error(ncli, "Read error: %s", strerror(errno));
    }

    return n;
}


static NCLI_STATUS
fd_write (NCLI ncli, char *buf, unsigned int len)
{
    FCD fcd = (FCD) ncli->client_data;
    
    return basic_fd_write(ncli, fcd->fdout, buf, len);
}

NCLI_STATUS
basic_fd_write(NCLI ncli, int fd, char *buf, size_t len)
{
    ssize_t n;
    while (len > 0)
	{
		n = write(fd, buf, len);
		if (n == -1)
		    return ncli->error(ncli, "Write error: %s",
				       strerror(errno));

		len -= n;
		buf += n;
    }
    return NCLI_SUCCESS;
}


static NCLI_STATUS
fd_flush (NCLI ncli)
{
    (void) ncli;

    return NCLI_SUCCESS;
}
