/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Framework for stackable network clients.  Actual clients are in
   socket-client.c, logging-client.c, ssl-client.c, etc.

*/

#include <assert.h>
#include <errno.h>
#include <stdlib.h>

#include "network-client.h"

NCLI
put_ncli_on_top_of_another (NCLI *top_ncli, NCLI new_ncli)
{
    new_ncli->next = *top_ncli;
    *top_ncli = new_ncli;
    return new_ncli;
}


/* Read a line using specified network client.  Result does not
   contain terminating LF and should be free()ed by caller.  Very dumb
   implementation. */

char *
ncli_read_line (NCLI ncli)
{
    int idx = 0;
    int result_len = 10;
    char *result = malloc(result_len);
    if (result == NULL)
	return NULL;

    while (1) {
	char ch = 'X';
	int nread = ncli->read(ncli, &ch, 1);
	if (nread == -1) {
	    assert((errno == EAGAIN) || (errno == EINTR));
	    if ((errno == EAGAIN) || (errno == EINTR))
		continue;
	}

	if (nread == 0)
	    return (void *) ncli->error(ncli, "end of file from server (consult above messages if any)");

	result[idx++] = ch;
	if (idx + 1 >= result_len) {
	    result_len *= 2;
	    result = realloc(result, result_len);
	    if (result == NULL) {
		free(result);
		return NULL;
	    }
	}
	
	if (ch == '\x0a') {
	    break;
	}
    }

    result[idx - 1] = '\x00';

    return result;
}
