/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Framework for stackable network clients.  Actual clients are in
   socket-client.c, logging-client.c, ssl-client.c, etc.

*/

#ifndef NETWORK_CLIENT_H
#define NETWORK_CLIENT_H

#include "cvsroot.h"

#define NCLI_FAILURE 0
#define NCLI_SUCCESS 1

typedef int NCLI_STATUS;

struct network_client;

/* error callbacks must always return NCLI_FAILURE as it is required
   by their common usage style */
typedef NCLI_STATUS (*ncli_error_cb) (struct network_client *, char *msg, ...);

struct network_client {
    NCLI_STATUS (*connect) (struct network_client *);
    NCLI_STATUS (*disconnect) (struct network_client *);

    int (*read) (struct network_client *, char *buf, unsigned int len);
    NCLI_STATUS (*write) (struct network_client *, char *buf, unsigned int len);
    NCLI_STATUS (*flush) (struct network_client *);

    ncli_error_cb error;

    struct network_client *next;
    void *client_data;
};

typedef struct network_client *NCLI;


/* Commonly-used routines */

NCLI put_ncli_on_top_of_another (NCLI *top_ncli, NCLI new_ncli);

char *ncli_read_line (NCLI ncli);

#endif /* NETWORK_CLIENT_H */
